#include "polybius.h"


int cryptPolybius(const char *filenameSrc, const char *filenameDst, const char *key){
    char polybius[POLYBIUS_SIZE][POLYBIUS_SIZE];
    FILE *f = NULL, *fDst = NULL;
    char data[BUFFER_SIZE];
    char dataEncrypted[BUFFER_SIZE];
    int size = 0, end = 0, i = 0, j = 0;
    char c = 0;

    generateSquarePolybius(polybius, key);

    for (i = 0; i < POLYBIUS_SIZE; i++){
        for (j = 0; j < POLYBIUS_SIZE; j++)
            printf("%c ", polybius[i][j]);
        printf("\n");
    }
    // Open the file
    f = fopen(filenameSrc, "r+");
    if (f == NULL) return -1;

    fDst = fopen(filenameDst, "w");
    if (fDst == NULL) {
        fclose(f);
        return -1;
    }
    do {
        end = readFile(f, &size, data);

	    /********* Encryption **********/
        for (i = 0; i < size; i++){
            if (data[i] >= 'a' && data[i] <= 'z')
                c = data[i] - 32;
            else c = data[i];

            for (j = 0; j < POLYBIUS_SIZE; j++){
                for (int x = 0; x < POLYBIUS_SIZE; x++){
                    if (c == polybius[j][x]){
                       fputc(j + '0', fDst);
                       fputc(x + '0', fDst);
                    }
                }
            }
            if (c == '\n' || c == ' '){
                fputc(c, fDst);
                fputc(0, fDst); // For having a couple of number
            }
        }

        size = 0; 
        memset(data, 0, BUFFER_SIZE);
        memset(dataEncrypted, 0, BUFFER_SIZE);
    }while(end != 1);

	/* Close files */
    fclose(f);
    fclose(fDst);
    return 0;
}
int decryptPolybius(const char *filenameSrc, const char *filenameDst, const char *key){
    char polybius[POLYBIUS_SIZE][POLYBIUS_SIZE];
    FILE *f = NULL, *fDst = NULL;
    char data[BUFFER_SIZE];
    char dataEncrypted[BUFFER_SIZE];
    int size = 0, end = 0, i = 0, j = 0;

    generateSquarePolybius(polybius, key);

    for (i = 0; i < POLYBIUS_SIZE; i++){
        for (j = 0; j < POLYBIUS_SIZE; j++)
            printf("%c ", polybius[i][j]);
        printf("\n");
    }
    // Open the file
    f = fopen(filenameSrc, "r+");
    if (f == NULL) return -1;

    fDst = fopen(filenameDst, "w");
    if (fDst == NULL) {
        fclose(f);
        return -1;
    }
    do {
        end = readFile(f, &size, data);

	    /********* Decryption **********/
        for (i = 0; i < size; i += 2){
            if (data[i] >= 48 && data[i] <= 53) {
                int c1 = data[i] - '0';
                int c2 = data[i+1] - '0';
                char c = polybius[c1][c2];
                printf("%c", c);
                fputc(c, fDst);
            }
            else{
                printf("%c", data[i]);
                fputc(data[i], fDst);
            }
        }

        size = 0; 
        memset(data, 0, BUFFER_SIZE);
        memset(dataEncrypted, 0, BUFFER_SIZE);
    }while(end != 1);

	/* Close files */
    fclose(f);
    fclose(fDst);
    return 0;

}
/*
 This function fill the polybius square
 */
static void fillSquare(int pos, int *posPolybius, char *p, char polybius[POLYBIUS_SIZE][POLYBIUS_SIZE]){
    for (int i = 0; i < POLYBIUS_SIZE; i++){
        polybius[pos][i] = p[*(posPolybius)];
        *posPolybius = *posPolybius + 1;
    }
}
static int findKeyInSquare(const char *polybius, char c){
    // We exclude the 'J' character
    for (int i = 0; i < ALPHABET_SIZE - 1; i++)
        if (c == polybius[i])
            return 1;
    return 0;
}
/*
 This function generate the polybius square
 */
void generateSquarePolybius(char polybius[POLYBIUS_SIZE][POLYBIUS_SIZE], const char *key){
    char a[ALPHABET_SIZE - 1] = {0}; // The size is 25, we exclude the J character

    int pos = 0;
    for (int i = 0; i < strlen(key); i++){
        char k;
        if (key[i] >= 'a' && key[i] <= 'z')
            k = key[i] - 32; // 32 = 26 + special characters
        else k = key[i];
        int res = findKeyInSquare(a, k);
        if (res == 0)
            a[pos++] = k;
    }
    int n = 0;
    int t = pos;
    for (int i = pos; i < ALPHABET_SIZE + pos; i++){
        char c = 'A' + n;
        if (c == 'J'){
            // We check if I is in the array, if yes, we do not add it
        }
        /*if (c == 'I'){
            // We check if J is in the array, is yes, we do not add it
            int res = findKeyInSquare(a, 'J');
            // J not in the array, we add it
            if (res == 0){
                a[t++] = 'A';
            }
        }*/
        if (c != 'J'){
            int res = findKeyInSquare(a, c);

            if (res == 0)
                a[t++] = c;
        }
        n++;
    }
    pos = 0;
    fillSquare(0, &pos, a, polybius);     
    fillSquare(1, &pos, a, polybius);     
    fillSquare(2, &pos, a, polybius);     
    fillSquare(3, &pos, a, polybius);     
    fillSquare(4, &pos, a, polybius);     
}
